@@ -27,6 +27,9 @@ ealgo=""
 ekey=""
 ekeyfile=""
 encrypt=0
+
+xbstream_opts=""
+
 nproc=1
 ecode=0
 ssyslog=""
@@ -292,8 +295,21 @@ get_transfer()
                 tcmd="nc $ncsockopt -dl ${TSST_PORT}"
             fi
         else
+            # Check to see if netcat supports the '-N' flag.
+            #      -N Shutdown the network socket after EOF on stdin
+            # If it supports the '-N' flag, then we need to use the '-N'
+            # flag, otherwise the transfer will stay open after the file
+            # transfer and cause the command to timeout.
+            # Older versions of netcat did not need this flag and will
+            # return an error if the flag is used.
+            #
+            tcmd_extra=""
+            if nc -h 2>&1 | grep -qw -- -N; then
+                tcmd_extra+=" -N "
+            fi
+
             # netcat doesn't understand [] around IPv6 address
-            tcmd="nc ${REMOTEIP//[\[\]]/} ${TSST_PORT}"
+            tcmd="nc ${tcmd_extra} ${REMOTEIP//[\[\]]/} ${TSST_PORT}"
         fi
     else
         tfmt='socat'
@@ -548,6 +564,7 @@ read_cnf()
         sockopt+=",retry=30"
     fi
 
+    xbstream_opts=$(parse_cnf sst xbstream-opts "")
 }
 
 #
@@ -561,9 +578,9 @@ get_stream()
     if [[ $sfmt == 'xbstream' ]];then 
         wsrep_log_info "Streaming with xbstream"
         if [[ "$WSREP_SST_OPT_ROLE"  == "joiner" ]];then
-            strmcmd="xbstream -x"
+            strmcmd="xbstream -x $xbstream_opts"
         else
-            strmcmd="xbstream -c \${FILE_TO_STREAM}"
+            strmcmd="xbstream -c $xbstream_opts \${FILE_TO_STREAM}"
         fi
     else
         sfmt="tar"
@@ -767,7 +784,7 @@ wait_for_listen()
 
             # Now get the processtree for this pid
             # If the parentpid is NOT in the process tree, then ignore
-            if ! echo $(get_parent_pids $pid) | grep -q " $parentpid "; then
+            if ! echo "$(get_parent_pids $pid)" | grep -qw "$parentpid"; then
                 continue
             fi
 
