@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 #include <my_global.h>
 #include <violite.h>
@@ -188,7 +188,6 @@ static int  create_worker(thread_group_t *thread_group);
 static void *worker_main(void *param);
 static void check_stall(thread_group_t *thread_group);
 static void connection_abort(connection_t *connection);
-void tp_post_kill_notification(THD *thd);
 static void set_wait_timeout(connection_t *connection);
 static void set_next_timeout_check(ulonglong abstime);
 static void print_pool_blocked_message(bool);
@@ -437,16 +436,23 @@ inline bool too_many_busy_threads(thread_group_t *thread_group)
 /*
    Checks if a given connection is eligible to enter the high priority queue
    based on its current thread_pool_high_prio_mode value, available high
-   priority tickets and transactional state.
+   priority tickets and transactional state and whether any locks are held.
 */
 
-inline bool connection_is_high_prio(connection_t *c)
+inline bool connection_is_high_prio(const connection_t *c)
 {
   const ulong mode= c->thd->variables.threadpool_high_prio_mode;
 
   return (mode == TP_HIGH_PRIO_MODE_STATEMENTS) ||
-    (mode == TP_HIGH_PRIO_MODE_TRANSACTIONS &&
-     c->tickets > 0 && thd_is_transaction_active(c->thd));
+    (mode == TP_HIGH_PRIO_MODE_TRANSACTIONS && c->tickets > 0 &&
+     (thd_is_transaction_active(c->thd) ||
+      c->thd->variables.option_bits & OPTION_TABLE_LOCK ||
+      c->thd->locked_tables_mode != LTM_NONE ||
+      c->thd->mdl_context.has_locks() ||
+      c->thd->global_read_lock.is_acquired() ||
+      c->thd->backup_tables_lock.is_acquired() ||
+      c->thd->backup_binlog_lock.is_acquired() ||
+      c->thd->ull_hash.records > 0));
 }
 
 } // namespace
@@ -563,7 +569,7 @@ static void* timer_thread(void *param)
     struct timespec ts;
     int err;
 
-    set_timespec_nsec(ts,timer->tick_interval*1000000);
+    set_timespec_nsec(ts,timer->tick_interval * 1000000ULL);
     mysql_mutex_lock(&timer->mutex);
     err= mysql_cond_timedwait(&timer->cond, &timer->mutex, &ts);
     if (timer->shutdown)
@@ -1053,6 +1059,7 @@ static void thread_group_close(thread_group_t *thread_group)
 
   if (pipe(thread_group->shutdown_pipe))
   {
+    mysql_mutex_unlock(&thread_group->mutex);
     DBUG_VOID_RETURN;
   }
   
@@ -1060,11 +1067,15 @@ static void thread_group_close(thread_group_t *thread_group)
   if (io_poll_associate_fd(thread_group->pollfd, 
       thread_group->shutdown_pipe[0], NULL))
   {
+    mysql_mutex_unlock(&thread_group->mutex);
     DBUG_VOID_RETURN;
   }
   char c= 0;
   if (write(thread_group->shutdown_pipe[1], &c, 1) < 0)
+  {
+    mysql_mutex_unlock(&thread_group->mutex);
     DBUG_VOID_RETURN;
+  }
 
   /* Wake all workers. */
   while(wake_thread(thread_group) == 0) 
@@ -1459,7 +1470,7 @@ static void set_wait_timeout(connection_t *c)
 
   c->abs_wait_timeout= pool_timer.current_microtime +
     1000LL*pool_timer.tick_interval +
-    1000000LL*c->thd->variables.net_wait_timeout;
+    1000000LL*c->thd->get_wait_timeout();
 
   set_next_timeout_check(c->abs_wait_timeout);
   DBUG_VOID_RETURN;
@@ -1766,26 +1777,29 @@ A likely cause of pool blocks are clients that lock resources for long time. \
 
 static void print_pool_blocked_message(bool max_threads_reached)
 {
-  ulonglong now;
+  ulonglong now= my_microsecond_getsystime();
   static bool msg_written;
-  
-  now= my_microsecond_getsystime();
+
   if (pool_block_start == 0)
   {
     pool_block_start= now;
-    msg_written = false;
-    return;
+    msg_written= false;
   }
-  
-  if (now > pool_block_start + BLOCK_MSG_DELAY && !msg_written)
+
+  if (!msg_written
+      && ((now > pool_block_start + BLOCK_MSG_DELAY)
+          || (now == pool_block_start)))
   {
     if (max_threads_reached)
       sql_print_error(MAX_THREADS_REACHED_MSG);
     else
       sql_print_error(CREATE_THREAD_ERROR_MSG, my_errno);
-    
-    sql_print_information("Threadpool has been blocked for %u seconds\n",
-      (uint)((now- pool_block_start)/1000000));
+
+    if (now > pool_block_start)
+    {
+      sql_print_information("Threadpool has been blocked for %u seconds\n",
+                            (uint)((now - pool_block_start)/1000000));
+    }
     /* avoid reperated messages for the same blocking situation */
     msg_written= true;
   }
