@@ -38,12 +38,16 @@ enum wsrep_trx_status wsrep_run_wsrep_commit(THD *thd, handlerton *hton,
 */
 void wsrep_cleanup_transaction(THD *thd)
 {
+  if (!WSREP(thd)) return;
+  DBUG_ASSERT(thd->wsrep_conflict_state != MUST_REPLAY &&
+              thd->wsrep_conflict_state != REPLAYING);
   if (wsrep_emulate_bin_log) thd_binlog_trx_reset(thd);
   thd->wsrep_ws_handle.trx_id= WSREP_UNDEFINED_TRX_ID;
   thd->wsrep_trx_meta.gtid= WSREP_GTID_UNDEFINED;
   thd->wsrep_trx_meta.depends_on= WSREP_SEQNO_UNDEFINED;
   thd->wsrep_exec_mode= LOCAL_STATE;
-  thd->wsrep_certify_empty_trx= false;
+  thd->wsrep_affected_rows= 0;
+  thd->wsrep_skip_wsrep_GTID= false;
   return;
 }
 
@@ -70,6 +74,15 @@ handlerton *wsrep_hton;
 */
 void wsrep_register_hton(THD* thd, bool all)
 {
+  /* Skip wsrep-hton handler registration.
+  This happens in case like following:
+  - DDL is executed with sql_log_bin = 0 where-in ddl replication is skipped
+    so the exec_mode != TOTAL_ORDER but as thumb rule hton should get registered
+    for DDL/TOI statement.
+  */
+  if (WSREP(thd) && thd->wsrep_skip_wsrep_hton)
+    return;
+
   if (WSREP(thd) && thd->wsrep_exec_mode != TOTAL_ORDER && !thd->wsrep_apply_toi)
   {
     if (thd->wsrep_exec_mode == LOCAL_STATE      &&
@@ -112,6 +125,8 @@ void wsrep_register_hton(THD* thd, bool all)
  */
 void wsrep_post_commit(THD* thd, bool all)
 {
+  if (!WSREP(thd)) return;
+
   switch (thd->wsrep_exec_mode)
   {
   case LOCAL_COMMIT:
@@ -131,9 +146,28 @@ void wsrep_post_commit(THD* thd, bool all)
      /* non-InnoDB statements may have populated events in stmt cache 
 	=> cleanup 
      */
-     WSREP_DEBUG("cleanup transaction for LOCAL_STATE: %s",
-                 WSREP_QUERY(thd));
-     wsrep_cleanup_transaction(thd);
+     if (thd->wsrep_conflict_state != MUST_REPLAY)
+     {
+       WSREP_DEBUG("cleanup transaction for LOCAL_STATE: %s",
+                   WSREP_QUERY(thd));
+     }
+     /*
+       Run post-rollback hook to clean up in the case if
+       some keys were populated for the transaction in provider
+       but during commit time there was no write set to replicate.
+       This may happen when client sets the SAVEPOINT and immediately
+       rolls back to savepoint after first operation. 
+      */
+     if (all && thd->wsrep_conflict_state != MUST_REPLAY &&
+         wsrep->post_rollback(wsrep, &thd->wsrep_ws_handle))
+     {
+         WSREP_WARN("post_rollback fail: %llu %d",
+                    (long long)thd->thread_id, thd->get_stmt_da()->status());
+     }
+     if (thd->wsrep_conflict_state != MUST_REPLAY)
+     {
+       wsrep_cleanup_transaction(thd);
+     }
      break;
    }
   default: break;
@@ -311,14 +345,12 @@ wsrep_run_wsrep_commit(THD *thd, handlerton *hton, bool all)
   int replay_round= 0;
 
   if (thd->get_stmt_da()->is_error()) {
-    WSREP_ERROR("commit issue, error: %d %s",
+    WSREP_DEBUG("commit issue, error: %d %s",
                 thd->get_stmt_da()->sql_errno(), thd->get_stmt_da()->message());
   }
 
   DBUG_ENTER("wsrep_run_wsrep_commit");
-#ifdef WITH_WSREP
   DEBUG_SYNC(thd, "wsrep_before_replication");
-#endif /* WITH_WSREP */
 
   if (thd->slave_thread && !opt_log_slave_updates) DBUG_RETURN(WSREP_TRX_OK);
 
@@ -441,7 +473,8 @@ wsrep_run_wsrep_commit(THD *thd, handlerton *hton, bool all)
                   "wsrep status (%d %d %d)",
                   WSREP_QUERY(thd),
                   thd->get_stmt_da()->affected_rows(),
-                  stmt_has_updated_trans_table(thd), thd->variables.sql_log_bin,
+                  stmt_has_updated_trans_table(thd->transaction.stmt.ha_list),
+                  thd->variables.sql_log_bin,
                   thd->wsrep_exec_mode, thd->wsrep_query_state,
                   thd->wsrep_conflict_state);
     }
@@ -503,6 +536,8 @@ wsrep_run_wsrep_commit(THD *thd, handlerton *hton, bool all)
     DBUG_RETURN(WSREP_TRX_ERROR);
   }
 
+  DEBUG_SYNC(thd, "wsrep_after_replication");
+
   mysql_mutex_lock(&thd->LOCK_wsrep_thd);
   switch(rcode) {
   case 0:
@@ -513,7 +548,10 @@ wsrep_run_wsrep_commit(THD *thd, handlerton *hton, bool all)
       set to NO_CONFLICT and commit proceeds as usual.
     */
     if (thd->wsrep_conflict_state == MUST_ABORT)
-        thd->wsrep_conflict_state= NO_CONFLICT;
+    {
+      thd->killed= THD::NOT_KILLED;
+      thd->wsrep_conflict_state= NO_CONFLICT;
+    }
 
     if (thd->wsrep_conflict_state != NO_CONFLICT)
     {
@@ -578,6 +616,38 @@ wsrep_run_wsrep_commit(THD *thd, handlerton *hton, bool all)
   DBUG_RETURN(WSREP_TRX_OK);
 }
 
+bool wsrep_replicate_GTID(THD *thd)
+{
+  if (thd->slave_thread)
+  {
+    WSREP_DEBUG("GTID replication");
+    DBUG_ASSERT (WSREP_UNDEFINED_TRX_ID == thd->wsrep_ws_handle.trx_id);
+    (void)wsrep_ws_handle_for_trx(&thd->wsrep_ws_handle, thd->query_id);
+    DBUG_ASSERT (WSREP_UNDEFINED_TRX_ID != thd->wsrep_ws_handle.trx_id);
+    WSREP_DEBUG("slave trx using query ID %llu for replication GTID",
+                (ulonglong) thd->wsrep_ws_handle.trx_id);
+    enum wsrep_trx_status rcode= wsrep_run_wsrep_commit(thd, wsrep_hton, true);
+    if (rcode)
+    {
+      /*
+        TODO: should error here cause stopping of MySQL slave?
+        Slave applying was totally filtered out, and fauílure in replicating
+        GTID event, would cause a hole in GTID history in other cluster nodes
+
+      */
+      WSREP_INFO("GTID replication failed: %d", rcode);
+      wsrep->post_rollback(wsrep, &thd->wsrep_ws_handle);
+      thd->wsrep_replicate_GTID= false;
+      my_message(ER_ERROR_DURING_COMMIT,
+                 "WSREP GTID replication was interrupted", MYF(0));
+      return true;
+    }
+    wsrep_post_commit(thd, true);
+  }
+  thd->wsrep_replicate_GTID= false;
+
+  return false;
+}
 
 static int wsrep_hton_init(void *p)
 {
