@@ -210,6 +210,32 @@ void wsrep_start_position_init (const char* val)
   wsrep_set_local_position (val, false);
 }
 
+static int get_provider_option_value(const char* opts,
+                                     const char* opt_name,
+                                     ulong* opt_value)
+{
+  int ret= 1;
+  ulong opt_value_tmp;
+  char *opt_value_str, *s, *opts_copy= my_strdup(opts, MYF(MY_WME));
+
+  if ((opt_value_str= strstr(opts_copy, opt_name)) == NULL)
+    goto end;
+  opt_value_str= strtok_r(opt_value_str, "=", &s);
+  if (opt_value_str == NULL) goto end;
+  opt_value_str= strtok_r(NULL, ";", &s);
+  if (opt_value_str == NULL) goto end;
+
+  opt_value_tmp= strtoul(opt_value_str, NULL, 10);
+  if (errno == ERANGE) goto end;
+
+  *opt_value= opt_value_tmp;
+  ret= 0;
+
+end:
+  my_free(opts_copy);
+  return ret;
+}
+
 static bool refresh_provider_options()
 {
   WSREP_DEBUG("refresh_provider_options: %s", 
@@ -217,9 +243,11 @@ static bool refresh_provider_options()
   char* opts= wsrep->options_get(wsrep);
   if (opts)
   {
-    if (wsrep_provider_options) my_free((void *)wsrep_provider_options);
-    wsrep_provider_options = (char*)my_memdup(opts, strlen(opts) + 1, 
-                                              MYF(MY_WME));
+    wsrep_provider_options_init(opts);
+    get_provider_option_value(wsrep_provider_options,
+                              (char*)"repl.max_ws_size",
+                              &wsrep_max_ws_size);
+    free(opts);
   }
   else
   {
@@ -256,27 +284,6 @@ static int wsrep_provider_verify (const char* provider_str)
 @return false if no error encountered with check else return true. */
 bool wsrep_provider_check (sys_var *self, THD* thd, set_var* var)
 {
-  if (WSREP(thd))
-  {
-    mysql_mutex_lock(&LOCK_wsrep_pause_count);
-    bool node_paused = (wsrep_pause_count == 0) ? false : true;
-    bool desynced = !wsrep_node_is_synced();
-    mysql_mutex_unlock(&LOCK_wsrep_pause_count);
-
-    if (node_paused || desynced)
-    {
-      /* If node is paused or desycned this means node is up-to-data.
-      Pause node hasn't applied all the write-set and desycned node
-      may have sent flow control. Avoid changing wsrep_provider
-      in such critical conditions. */
-      WSREP_WARN("Cannot modify wsrep_provider while node is paused"
-                 " or desynced.");
-      my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), var->var->name.str,
-               var->save_result.string_value.str);
-      return true;
-    }
-  }
-
   char wsrep_provider_buf[FN_REFLEN];
 
   if ((! var->save_result.string_value.str) ||
@@ -322,6 +329,14 @@ bool wsrep_provider_update (sys_var *self, THD* thd, enum_var_type type)
   bool wsrep_on_saved= thd->variables.wsrep_on;
   thd->variables.wsrep_on= false;
 
+  /* Ensure we free the stats that are allocated by galera-library.
+  wsrep part doesn't know how to free them shouldn't be attempting it
+  as it not allocated by wsrep part.
+  Before unloading cleanup any stale reference and stats is one of it.
+  Given that thd->variables.wsrep.on is turned-off it is safe to assume
+  that no new stats queries will be allowed during this transition time. */
+  wsrep_free_status(thd);
+
   WSREP_DEBUG("wsrep_provider_update: %s", wsrep_provider);
 
   /* stop replication is heavy operation, and includes closing all client 
@@ -333,6 +348,7 @@ bool wsrep_provider_update (sys_var *self, THD* thd, enum_var_type type)
   */
   mysql_mutex_unlock(&LOCK_global_system_variables);
   wsrep_stop_replication(thd);
+
   /*
     Unlock and lock LOCK_wsrep_slave_threads to maintain lock order & avoid
     any potential deadlock.
@@ -341,6 +357,14 @@ bool wsrep_provider_update (sys_var *self, THD* thd, enum_var_type type)
   mysql_mutex_lock(&LOCK_global_system_variables);
   mysql_mutex_lock(&LOCK_wsrep_slave_threads);
 
+#if 0
+  /* provider status variables are allocated in provider library
+     and need to freed here, otherwise a dangling reference to
+     wsrep_status_vars would remain in THD
+  */
+  wsrep_free_status(thd);
+#endif
+
   wsrep_deinit();
 
   char* tmp= strdup(wsrep_provider); // wsrep_init() rewrites provider 
@@ -582,18 +606,15 @@ void wsrep_node_address_init (const char* value)
   wsrep_node_address = (value) ? my_strdup(value, MYF(0)) : NULL;
 }
 
-bool wsrep_slave_threads_check (sys_var *self, THD* thd, set_var* var)
+static void wsrep_slave_count_change_update ()
 {
-  mysql_mutex_lock(&LOCK_wsrep_slave_threads);
-  wsrep_slave_count_change += (var->save_result.ulonglong_value -
-                               wsrep_slave_threads);
-  mysql_mutex_unlock(&LOCK_wsrep_slave_threads);
-
-  return false;
+  // wsrep_running_threads = appliers threads + 1 rollbacker thread
+  wsrep_slave_count_change = (wsrep_slave_threads - wsrep_running_threads + 1);
 }
 
 bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
 {
+  wsrep_slave_count_change_update();
   if (wsrep_slave_count_change > 0)
   {
     WSREP_DEBUG("Creating %d applier threads, total %ld", wsrep_slave_count_change, wsrep_slave_threads);
@@ -618,6 +639,22 @@ bool wsrep_desync_check (sys_var *self, THD* thd, set_var* var)
     }
     return false;
   }
+
+  /* Setting desync to on/off signals participation of node in flow control.
+  It doesn't turn-off recieval of write-sets.
+  If the node is already in paused state due to some previous action like FTWRL
+  then avoid desync as superset action of pausing the node is already active. */
+
+  if (!thd->global_read_lock.provider_resumed())
+  {
+    char message[1024];
+    sprintf(message,
+            "Explictly desync/resync of already desynced/paused node"
+            " is prohibited");
+    my_message(ER_UNKNOWN_ERROR, message, MYF(0));
+    return true;
+  }
+
   wsrep_status_t ret(WSREP_WARNING);
   if (new_wsrep_desync) {
     ret = wsrep->desync (wsrep);
@@ -644,6 +681,21 @@ bool wsrep_desync_update (sys_var *self, THD* thd, enum_var_type type)
   return false;
 }
 
+bool wsrep_max_ws_size_update (sys_var *self, THD *thd, enum_var_type)
+{
+  char max_ws_size_opt[128];
+  my_snprintf(max_ws_size_opt, sizeof(max_ws_size_opt),
+              "repl.max_ws_size=%d", wsrep_max_ws_size);
+  wsrep_status_t ret= wsrep->options_set(wsrep, max_ws_size_opt);
+  if (ret != WSREP_OK)
+  {
+    WSREP_ERROR("Set options returned %d", ret);
+    refresh_provider_options();
+    return true;
+  }
+  return refresh_provider_options();
+}
+
 /*
  * Status variables stuff below
  */
@@ -680,6 +732,10 @@ static void export_wsrep_status_to_mysql(THD* thd)
 {
   int wsrep_status_len, i;
 
+  /* Avoid freeing the stats immediately on completion of the call
+  instead free them on next invocation as the reference to status
+  is feeded in MySQL show status array. Freeing them on completion
+  will make these references invalid. */
   wsrep_free_status(thd);
 
   thd->wsrep_status_vars = wsrep->stats_get(wsrep);
